// Simple Wekinator color grid extractor
// Features are the average hue of each square in an NxN grid (default 3)
// Run as an OSC extractor with NxN features (default = 3x3 = 9 custom OSC features)
// by Rebecca Fiebrink

import processing.video.*;
import oscP5.*;
import netP5.*;

int gridSize = 3; //default: a 3x3 grid of colors (9 features)
float[] hues = new float[gridSize * gridSize];
int fps = 15;
int numPixels;
Capture video;

OscP5 oscP5;
NetAddress dest;
float xscl, yscl;
      
void setup() {
  size(640, 480, P2D);
  colorMode(HSB, 100);
  video = new Capture(this, gridSize, gridSize, fps);
  numPixels = video.width * video.height;
  loadPixels();
  noStroke();
  
  xscl = (float) width / (float) video.width;
  yscl = (float) height / (float) video.height;

  /* start oscP5, listening for incoming messages at port 12000 */
  oscP5 = new OscP5(this,12000);
  dest = new NetAddress("127.0.0.1",6448); //send to Wekinator on port 6448 (don't change), on localhost (could change if desired)
  sendNames(); //Send feature names to Wekinator for display
}

void draw() {
  if (video.available()) {
    video.read(); // Read a new video frame
    video.loadPixels(); // Make the pixels of video available
    for (int i = 0; i < numPixels; i++) {
      int x = i % video.width;
      int y = i / video.width;
      fill(video.pixels[i]);
      rect(x * xscl, y * yscl, xscl, yscl);
    }
  }
  
  if(frameCount % 5 == 0) //Send every 5 frames for efficiency (optional)
    sendOsc(video.pixels);
}



//Send feature names to Wekinator so they're displayed nicely in GUI
//Only has to be called once (after Wekinator is launched), but it doesn't hurt if it's called more than once
void sendNames() {
  OscMessage msg = new OscMessage("/oscCustomFeaturesNames");
  for (int i = 0; i < gridSize * gridSize; i++) {
     msg.add("hueCell" + i);   
  }
  oscP5.send(msg, dest); 
}


void sendOsc(int[] px) {
  OscMessage msg = new OscMessage("/oscCustomFeatures");
  for (int i = 0; i < px.length; i++) {
    msg.add((float)hue(px[i]));    
  }
  oscP5.send(msg, dest);
}
